import numpy as np
import rclpy
from rclpy.node import Node
from sensor_msgs.msg import LaserScan
import geometry_msgs.msg as gm


class TF2PublisherNode(Node):
    """ The purpose of this node is to modify the LaserScan produced by velodyne so as to be compatible with slam_toolbox.
        It seems there is an off-by-one error in slam_toolbox calculating the number of expected points in the scan,
        resulting in warnings that /scan contains fewer points than expected.
    """
    def __init__(self):
        super().__init__('nanhandler')
        
        self.create_subscription(LaserScan, "/scan", self.callback, 1)
        self.pub = self.create_publisher(LaserScan, "/scan_no_nan", 1)
    def callback(self, msg):
        print(msg.ranges[0])
        
        msg.ranges[0] = float("inf")
        msg.intensities[0] = 0.0

        msg.ranges.append(float("inf"))
        msg.intensities.append(0.0)
        
        self.pub.publish(msg)



def main(args=None):
    rclpy.init(args=args)

    tf2_publisher = TF2PublisherNode()
    rclpy.spin(tf2_publisher)

    tf2_publisher.destroy_node()
    rclpy.shutdown()


if __name__ == '__main__':
    main()